/* The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Mobile Application Link.
 *
 * The Initial Developer of the Original Code is AvantGo, Inc.
 * Portions created by AvantGo, Inc. are Copyright (C) 1997-1999
 * AvantGo, Inc. All Rights Reserved.
 *
 */

/* syncmal.c
 *
 * J-Pilot plugin callback functions by Jason Day, jasonday@worldnet.att.net
 * http://jasonday.home.att.net
 */




#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <utime.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <dirent.h>
#include <dlfcn.h>

#include <pi-dlp.h>
#include <pi-file.h>

#include "syncmal.h"
#include "libplugin.h"
#include "plugin_prefs.h"
#include "libmal.h"


#include <pi-source.h>
#include <pi-socket.h>
#include <pi-version.h>


static char *RCSID = "$Id: syncmal.c,v 1.15 2000/03/24 16:41:37 jday Exp $";
#define VERSION "0.43"

#define PREFS_FILE "syncmal.rc"

#define SMPREF_ENABLED 0
#define SMPREF_USE_PROXY 1
#define SMPREF_PROXY_ADDRESS 2
#define SMPREF_PROXY_PORT 3
#define SMPREF_PROXY_USERNAME 4
#define SMPREF_PROXY_PASSWORD 5
#define SMPREF_USE_SOCKS 6
#define SMPREF_SOCKS_ADDRESS 7
#define SMPREF_SOCKS_PORT 8

#define NUM_SMPREFS 9
prefType syncmal_prefs[NUM_SMPREFS] = {
    {"enabled", INTTYPE, INTTYPE, 1, ""},
    {"use_proxy", INTTYPE, INTTYPE, 0, ""},
    {"proxy_address", CHARTYPE, CHARTYPE, 0, ""},
    {"proxy_port", CHARTYPE, CHARTYPE, 0, ""},
    {"proxy_username", CHARTYPE, CHARTYPE, 0, ""},
    {"proxy_password", CHARTYPE, CHARTYPE, 0, ""},
    {"use_socks", INTTYPE, INTTYPE, 0, ""},
    {"socks_address", CHARTYPE, CHARTYPE, 0, ""},
    {"socks_port", CHARTYPE, CHARTYPE, 0, ""}
};

/*
 * These two arrays just hold pointers to some of the labels and text
 * entries. This makes it easier to enable/disable them and cuts down
 * on the number of global variables.
 */
#define PROXY_WIDGETS 8
static GtkWidget *proxy_widgets[PROXY_WIDGETS];
#define SOCKS_WIDGETS 4
static GtkWidget *socks_widgets[SOCKS_WIDGETS];


/* static function prototypes */
static int check_prefs_file();
static char *encode (char *str);
static char *decode (char *str);
static void cb_save (GtkWidget *widget, gpointer data);
static void cb_enabled (GtkWidget *widget, gpointer data);
static void cb_proxy_enabled (GtkWidget *widget, gpointer data);
static void cb_socks_enabled (GtkWidget *widget, gpointer data);
static void cb_proxy_address (GtkWidget *widget, GtkWidget *entry);
static void cb_proxy_port (GtkWidget *widget, GtkWidget *entry);
static void cb_proxy_username (GtkWidget *widget, GtkWidget *entry);
static void cb_proxy_password (GtkWidget *widget, GtkWidget *entry);
static void cb_socks_address (GtkWidget *widget, GtkWidget *entry);
static void cb_socks_port (GtkWidget *widget, GtkWidget *entry);




int plugin_get_name(char *name, int len)
{
    strncpy (name,
             "SyncMAL "
             VERSION,
             len);
    return 0;
}

int plugin_get_menu_name(char *name, int len)
{
    strncpy(name, "SyncMAL", len);
    return 0;
}

int plugin_get_help_name(char *name, int len)
{
    strncpy(name, "About SyncMAL", len);
    return 0;
}

int plugin_help(char **text, int *width, int *height)
{
    /* We could also pass back *text=NULL */
    *text = strdup(
                   /*-------------------------------------------*/
                   "SyncMAL plugin for J-Pilot\r\n"
                   "version " VERSION "\r\n"
                   "Jason Day (c) 2000.\r\n"
                   "jasonday@worldnet.att.net\r\n"
                   "http://jasonday.home.att.net\r\n"
                  );
    *height = 200;
    *width = 300;
    return 0;
}

/*
 * This function is used by libmal to update the GUI with status
 * messages.
 */
int status_hook (const char *format, ...) {
    va_list val;
    int     rval;

    va_start(val, format);
    rval = jpilot_vlogf (LOG_INFO | LOG_GUI, format, val);
    va_end(val);
    return rval;
}

int plugin_startup (jp_startup_info *info)
{
    int i;
    jp_init();
    jp_logf (LOG_DEBUG, "SyncMAL: plugin_startup\n");

    /* Open the prefs file */
    if (check_prefs_file() < 0) {
        jp_logf (LOG_FATAL,
                 "--------------------------------------------\n"
                 "ERROR: The preferences file " PREFS_FILE "\n"
                 "does not have the correct permissions and I\n"
                 "cannot change them. Please type\n"
                 "   chmod 0600 " PREFS_FILE "\n"
                 "in the ~/.jpilot directory to correct this.\n"
                 "--------------------------------------------\n");
    }
    if (jp_read_rc_file (PREFS_FILE, syncmal_prefs, NUM_SMPREFS) < 0) {
        jp_logf (LOG_WARN, "SyncMAL: Unable to load preferences file " PREFS_FILE "\n");
    }
    else {
        jp_logf (LOG_DEBUG, "SyncMAL: loaded preferences from " PREFS_FILE "\n");
    }

    jp_logf (LOG_DEBUG, "SyncMAL: registering print status hook...\n");
    register_printStatusHook (status_hook);
    jp_logf (LOG_DEBUG, "SyncMAL: set print status hook to status_hook\n");
    return 0;
}

int plugin_sync (int sd) {
    PalmSyncInfo * pInfo;
    long ivalue;
    const char *svalue;

    jp_logf (LOG_DEBUG, "SyncMAL: plugin_sync\n");

    /* return immediately if we're disabled */
    jp_get_pref (syncmal_prefs, SMPREF_ENABLED, &ivalue, &svalue);
    if (ivalue == 0) {
        jp_logf (LOG_INFO | LOG_GUI, "SyncMAL: skipping at user request\n");
        return 0;
    }

    /* check the prefs file again; refuse to work if the perms are wrong */
    if (check_prefs_file() < 0) {
        jp_logf (LOG_FATAL,
                 "--------------------------------------------\n"
                 "ERROR: The preferences file " PREFS_FILE "\n"
                 "does not have the correct permissions and I\n"
                 "cannot change them. Please type\n"
                 "   chmod 0600 " PREFS_FILE "\n"
                 "in the ~/.jpilot directory to correct this.\n"
                 "--------------------------------------------\n");
        return -1;
    }

    /* are we using a proxy? */
    jp_get_pref (syncmal_prefs, SMPREF_USE_PROXY, &ivalue, &svalue);
    if (ivalue) {
        jp_logf (LOG_DEBUG, "plugin_sync - using http proxy\n");

        jp_get_pref (syncmal_prefs, SMPREF_PROXY_ADDRESS, &ivalue, &svalue);
        if (svalue) {
            jp_logf (LOG_DEBUG, "plugin_sync - setting http proxy: %s\n", svalue);
            setHttpProxy ((char *)svalue);
        }
        else {
            jp_logf (LOG_FATAL,
                     "--------------------------------------------\n"
                     "ERROR: Proxy enabled but no proxy specified.\n"
                     "Please specify a proxy address or unselect\n"
                     "the Use Proxy checkbox.\n"
                     "--------------------------------------------\n");
            return 1;
        }

        jp_get_pref (syncmal_prefs, SMPREF_PROXY_PORT, &ivalue, &svalue);
        if (svalue) {
            jp_logf (LOG_DEBUG, "plugin_sync - setting http proxy port: %s\n", svalue);
            setHttpProxyPort (atoi (svalue));
        }
        else {
            jp_logf (LOG_INFO | LOG_GUI,
                     "SyncMAL: Using default proxy port 80\n");
            setHttpProxyPort (80);
        }

        jp_get_pref (syncmal_prefs, SMPREF_PROXY_USERNAME, &ivalue, &svalue);
        if (svalue) {
            if (strlen (svalue)) {
                jp_logf (LOG_DEBUG, "plugin_sync - setting proxy username: %s\n", svalue);
                setProxyUsername ((char *)svalue);
            }
        }

        jp_get_pref (syncmal_prefs, SMPREF_PROXY_PASSWORD, &ivalue, &svalue);
        if (svalue) {
            if (strlen (svalue)) {
                jp_logf (LOG_DEBUG, "plugin_sync - setting proxy password: xxxxxxxx\n");
                setProxyPassword ((char *)svalue);
            }
        }
    }

    /* are we using SOCKS? */
    jp_get_pref (syncmal_prefs, SMPREF_USE_SOCKS, &ivalue, &svalue);
    if (ivalue) {
        jp_get_pref (syncmal_prefs, SMPREF_SOCKS_ADDRESS, &ivalue, &svalue);
        if (svalue) {
            jp_logf (LOG_DEBUG, "plugin_sync - setting socks address: %s\n", svalue);
            setSocksProxy ((char *)svalue);
        }
        else {
            jp_logf (LOG_FATAL,
                     "----------------------------------------------\n"
                     "ERROR: SOCKS enabled but no address specified.\n"
                     "Please specify an address or unselect the\n"
                     "Use SOCKS checkbox.\n"
                     "----------------------------------------------\n");
            return 1;
        }

        jp_get_pref (syncmal_prefs, SMPREF_SOCKS_PORT, &ivalue, &svalue);
        if (svalue) {
            jp_logf (LOG_DEBUG, "plugin_sync - setting socks port: %s\n", svalue);
            setSocksProxyPort (atoi (svalue));
        }
        else {
            jp_logf (LOG_INFO | LOG_GUI,
                     "SyncMAL: Using default SOCKS port 80\n");
            setSocksProxyPort (80);
        }
    }

    pInfo = syncInfoNew();
    if (NULL == pInfo)
        return -1;

    malsync (sd, pInfo);

    syncInfoFree(pInfo);

    return 0;
}

/*
 * This function is called by J-Pilot when the user selects this plugin
 * from the plugin menu, or from the search window when a search result
 * record is chosen.  In the latter case, unique ID will be set.  This
 * application should go directly to that record in the case.
 */
int plugin_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
{
    GtkWidget *vbox1;
    GtkWidget *button;
    GtkWidget *temp_hbox;
    GtkWidget *label;
    GtkWidget *entry;
    GtkWidget *table;
    long ivalue;
    const char *svalue;

    jp_logf(LOG_DEBUG, "SyncMAL: plugin gui started\n");

    vbox1 = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX(hbox), vbox1, TRUE, FALSE, 0);

    /* ----------------------------------- */
    /* Top portion of screen (preferences) */
    /* ----------------------------------- */

    button = gtk_check_button_new_with_label ("Enabled");
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
                        GTK_SIGNAL_FUNC (cb_enabled), NULL);

    jp_get_pref (syncmal_prefs, SMPREF_ENABLED, &ivalue, &svalue);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), ivalue);
    gtk_box_pack_start (GTK_BOX(vbox1), button, FALSE, FALSE, 5);

    gtk_box_pack_start (GTK_BOX(vbox1), gtk_hseparator_new(), FALSE, FALSE, 0);

    /* --------------*/
    /* Proxy Options */
    /* --------------*/
    button = gtk_check_button_new_with_label ("Use Proxy");
    jp_get_pref (syncmal_prefs, SMPREF_USE_PROXY, &ivalue, &svalue);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), ivalue);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
                        GTK_SIGNAL_FUNC (cb_proxy_enabled), NULL);
    gtk_box_pack_start (GTK_BOX(vbox1), button, FALSE, FALSE, 5);


    table = gtk_table_new (4, 2, FALSE);

    label = gtk_label_new ("Proxy Address:");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    proxy_widgets[0] = label;
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
    entry = gtk_entry_new();
    proxy_widgets[1] = entry;
    jp_get_pref (syncmal_prefs, SMPREF_PROXY_ADDRESS, &ivalue, &svalue);
    if (svalue) {
        gtk_entry_set_text (GTK_ENTRY(entry), svalue);
    }
    gtk_signal_connect(GTK_OBJECT(entry), "changed",
                       GTK_SIGNAL_FUNC(cb_proxy_address),
                       entry);
    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 10, 0);


    label = gtk_label_new ("Proxy Port:");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    proxy_widgets[2] = label;
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
    entry = gtk_entry_new();
    proxy_widgets[3] = entry;
    jp_get_pref (syncmal_prefs, SMPREF_PROXY_PORT, &ivalue, &svalue);
    if (svalue) {
        gtk_entry_set_text (GTK_ENTRY(entry), svalue);
    }
    gtk_signal_connect(GTK_OBJECT(entry), "changed",
                       GTK_SIGNAL_FUNC(cb_proxy_port),
                       entry);
    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 10, 0);


    label = gtk_label_new ("Proxy Username:");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    proxy_widgets[4] = label;
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
    entry = gtk_entry_new();
    proxy_widgets[5] = entry;
    jp_get_pref (syncmal_prefs, SMPREF_PROXY_USERNAME, &ivalue, &svalue);
    if (svalue) {
        gtk_entry_set_text (GTK_ENTRY(entry), svalue);
    }
    gtk_signal_connect(GTK_OBJECT(entry), "changed",
                       GTK_SIGNAL_FUNC(cb_proxy_username),
                       entry);
    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 10, 0);


    label = gtk_label_new ("Proxy Password:");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    proxy_widgets[6] = label;
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 3, 4);
    entry = gtk_entry_new();
    proxy_widgets[7] = entry;
    jp_get_pref (syncmal_prefs, SMPREF_PROXY_PASSWORD, &ivalue, &svalue);
    if (svalue) {
        gtk_entry_set_text (GTK_ENTRY(entry), svalue);
    }
    gtk_signal_connect(GTK_OBJECT(entry), "changed",
                       GTK_SIGNAL_FUNC(cb_proxy_password),
                       entry);
    gtk_entry_set_visibility (GTK_ENTRY(entry), FALSE);
    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 3, 4,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 10, 0);


    gtk_box_pack_start (GTK_BOX(vbox1), table, FALSE, FALSE, 0);
    cb_proxy_enabled (button, NULL);

    gtk_box_pack_start (GTK_BOX(vbox1), gtk_hseparator_new(), FALSE, FALSE, 5);

    /* --------------*/
    /* Socks Options */
    /* --------------*/
    button = gtk_check_button_new_with_label ("Use SOCKS");
    jp_get_pref (syncmal_prefs, SMPREF_USE_SOCKS, &ivalue, &svalue);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), ivalue);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
                        GTK_SIGNAL_FUNC (cb_socks_enabled), NULL);
    gtk_box_pack_start (GTK_BOX(vbox1), button, FALSE, FALSE, 5);

    table = gtk_table_new (2, 2, FALSE);

    label = gtk_label_new ("SOCKS Proxy:");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    socks_widgets[0] = label;
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
    entry = gtk_entry_new();
    socks_widgets[1] = entry;
    jp_get_pref (syncmal_prefs, SMPREF_SOCKS_ADDRESS, &ivalue, &svalue);
    if (svalue) {
        gtk_entry_set_text (GTK_ENTRY(entry), svalue);
    }
    gtk_signal_connect(GTK_OBJECT(entry), "changed",
                       GTK_SIGNAL_FUNC(cb_socks_address),
                       entry);
    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 10, 0);

    label = gtk_label_new ("SOCKS Port:");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    socks_widgets[2] = label;
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
    entry = gtk_entry_new();
    socks_widgets[3] = entry;
    jp_get_pref (syncmal_prefs, SMPREF_SOCKS_PORT, &ivalue, &svalue);
    if (svalue) {
        gtk_entry_set_text (GTK_ENTRY(entry), svalue);
    }
    gtk_signal_connect(GTK_OBJECT(entry), "changed",
                       GTK_SIGNAL_FUNC(cb_socks_port),
                       entry);
    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 10, 0);

    gtk_box_pack_start (GTK_BOX(vbox1), table, FALSE, FALSE, 5);
    cb_socks_enabled (button, NULL);

    /* ------------ */
    /* Apply button */
    /* ------------ */
    gtk_box_pack_start (GTK_BOX(vbox1), gtk_hseparator_new(), FALSE, FALSE, 5);

    button = gtk_button_new_with_label ("Save Changes");
    gtk_signal_connect (GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(cb_save), NULL);
    gtk_box_pack_start (GTK_BOX(vbox1), button, FALSE, FALSE, 5);


    gtk_widget_show_all (hbox);
    return 0;
}

/* ----------------------------------------------------------------- */

static int check_prefs_file() {
    char filename[256];
    struct stat statb;

    get_home_file_name (PREFS_FILE, filename, 255);

    if (stat (filename, &statb) == -1) {
        if (errno == ENOENT) {
            /* doesn't exist */
            return 0;
        }
        else {
            perror ("stat");
            return -1;
        }
    }

    if (statb.st_mode & ~(S_IRUSR & S_IWUSR)) {
        if (chmod (filename, S_IRUSR | S_IWUSR) == -1) {
            return -1;
        }
        jp_logf (LOG_INFO, "Permissions set on prefs file: %s\n", filename);
    }

    return 0;
}

/* ----------------------------------------------------------------- */
/* Callbacks */

static void cb_save (GtkWidget *widget, gpointer data) {
    jp_write_rc_file (PREFS_FILE, syncmal_prefs, NUM_SMPREFS);
    if (check_prefs_file() < 0) {
        jp_logf (LOG_FATAL,
                 "--------------------------------------------\n"
                 "ERROR: The preferences file " PREFS_FILE "\n"
                 "does not have the correct permissions and I\n"
                 "cannot change them. Please type\n"
                 "   chmod 0600 " PREFS_FILE "\n"
                 "in the ~/.jpilot directory to correct this.\n"
                 "--------------------------------------------\n");
    }
}

static void cb_enabled (GtkWidget *widget, gpointer data)
{
    gboolean enabled = GTK_TOGGLE_BUTTON(widget)->active;
    jp_set_pref_int (syncmal_prefs, SMPREF_ENABLED, enabled);
}

static void cb_proxy_enabled (GtkWidget *widget, gpointer data)
{
    int i;
    gboolean enabled = GTK_TOGGLE_BUTTON(widget)->active;

    jp_set_pref_int (syncmal_prefs, SMPREF_USE_PROXY, enabled);
    for (i = 0; i < PROXY_WIDGETS; i++) {
        gtk_widget_set_sensitive (proxy_widgets[i], enabled);
    }
}

static void cb_socks_enabled (GtkWidget *widget, gpointer data)
{
    int i;
    gboolean enabled = GTK_TOGGLE_BUTTON(widget)->active;

    jp_set_pref_int (syncmal_prefs, SMPREF_USE_SOCKS, enabled);
    for (i = 0; i < SOCKS_WIDGETS; i++) {
        gtk_widget_set_sensitive (socks_widgets[i], enabled);
    }
}

static void cb_proxy_address (GtkWidget *widget, GtkWidget *entry)
{
    char *httpProxy = gtk_entry_get_text(GTK_ENTRY(entry));
    jp_set_pref_char (syncmal_prefs, SMPREF_PROXY_ADDRESS, httpProxy);
}

static void cb_proxy_port (GtkWidget *widget, GtkWidget *entry)
{
    int httpProxyPort = atoi (gtk_entry_get_text(GTK_ENTRY(entry)));
    jp_set_pref (syncmal_prefs,
                 SMPREF_PROXY_PORT,
                 httpProxyPort,
                 gtk_entry_get_text(GTK_ENTRY(entry)));
}

static void cb_proxy_username (GtkWidget *widget, GtkWidget *entry)
{
    char *proxyUsername = gtk_entry_get_text(GTK_ENTRY(entry));
    jp_set_pref_char (syncmal_prefs, SMPREF_PROXY_USERNAME, proxyUsername);
}

static void cb_proxy_password (GtkWidget *widget, GtkWidget *entry)
{
    char *proxyPassword = gtk_entry_get_text(GTK_ENTRY(entry));
    jp_set_pref_char (syncmal_prefs, SMPREF_PROXY_PASSWORD, proxyPassword);
}

static void cb_socks_address (GtkWidget *widget, GtkWidget *entry)
{
    char *socksProxy = gtk_entry_get_text(GTK_ENTRY(entry));
}

static void cb_socks_port (GtkWidget *widget, GtkWidget *entry)
{
    int socksProxyPort = atoi (gtk_entry_get_text(GTK_ENTRY(entry)));
    jp_set_pref (syncmal_prefs,
                 SMPREF_SOCKS_PORT,
                 socksProxyPort,
                 gtk_entry_get_text (GTK_ENTRY(entry)));
}
